# -*- coding: utf-8 -*-
# @Author: JanKinCai
# @Date:   2019-11-09 14:57:10
# @Last Modified by:   JanKinCai
# @Last Modified time: 2020-04-04 11:59:02

__version__ = "0.2.4"
__author__ = "JanKinCai"
__all__ = [
    "rpcap",
    "wpcap",
    "mpcap",
    "sniff",
    "get_first_iface",
    "get_iface_list",
    "send_packet"
]


try:
    from pylibpcap.pcap import rpcap, wpcap, mpcap, sniff
    from pylibpcap.base import get_first_iface, get_iface_list, send_packet
    from pylibpcap.open import OpenPcap
except ImportError:
    pass



# -*- coding: utf-8 -*-
# @Author: JanKinCai
# @Date:   2019-09-11 17:43:19
# @Last Modified by:   jankincai12@gmail.com
# @Last Modified time: 2019-09-12 09:27:39
from pylibpcap.base import LibPcap


class OpenPcap(object):
    """
    OpenPcap
    """

    def __init__(self, path, mode="r", *args, **kwargs):
        """
        init

        :param path: file path.
        :param mode: file model, default ``r``
        """

        self._pcapobj = LibPcap(path, mode, *args, **kwargs)

    def read(self):
        """
        Read
        """

        return self._pcapobj.read()

    def write(self, v):
        """
        Write

        :param v: Payload
        """

        return self._pcapobj.write(v)

    def write_path(self, path):
        """
        write path
        """

        return self._pcapobj.write_path(path)

    def __enter__(self):
        """
        enter
        """

        return self

    def __exit__(self, exception_type, exception_value, traceback):
        """
        exit
        """

        self._pcapobj.close()



# -*- coding: utf-8 -*-
# @Author: JanKinCai
# @Date:   2019-05-10 11:46:33
# @Last Modified by:   jankincai12@gmail.com
# @Last Modified time: 2019-09-12 13:28:50
from pylibpcap.base import Sniff, LibPcap
from pylibpcap.open import OpenPcap


def rpcap(path, filters=""):
    """
    Read pcap file.

    :param path: 文件路径.
    :param filters: BPF Filters

    return tuple: (Buf length，Capture time，Buf)
    """

    return LibPcap(path, filters=filters).read()


def wpcap(v, path):
    """
    Write pcap file.

    :param v: Buf/Buf(list)
    :param out_file: Output file
    """

    with OpenPcap(path, "a") as f:
        f.write(v)


def mpcap(path, out_file, filters=""):
    """
    Merge many pcap file.

    :param path: Input dir/file
    :param out_file: Output file
    :param filters: BPF Filters, default ``""``
    """

    with OpenPcap(out_file, "a", filters=filters) as f:
        f.write_path(path)


def sniff(*args, **kwargs):
    """
    Capture packet

    :param iface: Iface
    :param count: Capture packet num, default ``-1``
    :param promisc: Promiscuous mode, default ``0``
    :param snaplen: Cut packet lenght, default ``65535``
    :param filters: BPF filter rules, default ``""``
    :param out_file: Output pcap file, default ``""``
    """

    return Sniff(*args, **kwargs).capture()



# -*- coding: utf-8 -*-
# @Author: JanKinCai
# @Date:   2019-09-03 09:50:27
# @Last Modified by:   JanKinCai
# @Last Modified time: 2020-01-13 15:38:43
import argparse

from pylibpcap.pcap import mpcap, sniff
from pylibpcap.open import OpenPcap


def main():
    """
    merge pcap file.
    """

    parser = argparse.ArgumentParser(description="Merge pcap file.")
    parser.add_argument("filter", nargs="*", type=str, help="BPF filter rules.")
    parser.add_argument("-i", "--input", type=str, help="Input file/path.", required=True)
    parser.add_argument("-o", "--output", type=str, help="Output file.", required=True)
    args = parser.parse_args()

    print("[+]:", args)

    mpcap(args.input, args.output, " ".join(args.filter))


def pylibpcap_sniff():
    """
    Capture Packet

    :param iface: Iface
    :param count: Capture packet num, default ``-1``
    :param promisc: Promiscuous mode, default ``0``
    :param snaplen: Cut packet lenght, default ``65535``
    :param filters: BPF filter rules, default ``""``
    :param out_file: Output pcap file, default ``""``
    """

    parser = argparse.ArgumentParser(description="Sniff")
    parser.add_argument("-i", "--iface", type=str, help="Iface", required=True)
    parser.add_argument("-c", "--count", type=int, default=-1, help="Capture packet num")
    parser.add_argument("-m", "--promisc", type=int, default=0, help="Promiscuous mode")
    parser.add_argument("filter", nargs="*", type=str, help="BPF filter rules")
    parser.add_argument("-o", "--output", type=str, help="Output pcap file")
    parser.add_argument("-v", "--view", action="store_true", help="是否显示")
    args = parser.parse_args()

    print("[+]:", args)

    num = 0

    try:
        for plen, t, buf in sniff(iface=args.iface, count=args.count, promisc=args.promisc,
                                  filters=" ".join(args.filter), out_file=args.output):
            num += 1

            if args.view:
                print("[+]: Payload len=", plen)
                print("[+]: Time", t)
                print("[+]: Payload", buf)
    except KeyboardInterrupt:
        pass

    print("\nPacket Count:", num)


def write_pcap_cli():
    """Write pcap cli
    """

    parser = argparse.ArgumentParser(description="Write pcap")
    parser.add_argument("-i", "--input", type=str, help="File path.")
    parser.add_argument("payload", nargs=1, type=str, help="Payload")
    args = parser.parse_args()

    path = args.input or "pcap.pcap"

    with OpenPcap(path, "a") as f:
        f.write(bytes.fromhex(args.payload[0]))



# -*- coding: utf-8 -*-
# @Author: JanKinCai
# @Date:   2019-09-04 23:19:03
# @Last Modified by:   JanKinCai
# @Last Modified time: 2019-11-20 10:04:12
import os


def to_c_str(v):
    """
    Python str to C str
    """

    try:
        return v.encode("utf-8")
    except Exception:
        pass

    return b""


def from_c_str(v):
    """
    C str to Python str
    """

    try:
        return v.decode("utf-8")
    except Exception:
        pass

    return ""


def get_pcap_file(path):
    """
    get pcap file

    :param path: path.
    """

    if isinstance(path, bytes):
        path = from_c_str(path)

    if os.path.isfile(path):
        return (path, )

    return (
        os.path.join(directory, file)
        for directory, dirs, files in os.walk(path)
        for file in files if ".pcap" in file
    )



# -*- coding: utf-8 -*-
# @Author: JanKinCai
# @Date:   2019-11-09 10:08:53
# @Last Modified by:   JanKinCai
# @Last Modified time: 2020-07-07 09:35:38
import os
import glob

from setuptools import setup, find_packages
from setuptools import Extension
from Cython.Build import cythonize
from Cython.Distutils import build_ext


with open('README.rst') as f:
    long_description = f.read()


def read_requirements(path):
    """
    递归读取requirements

    :param path: path
    """

    requires = []

    with open(path) as f:
        install_requires = f.read().split("\n")

        for ir in install_requires:
            if "-r" in ir:
                path = os.path.join(os.path.split(path)[0], ir.split(" ")[1])
                requires.extend(read_requirements(path))
            else:
                ir and requires.append(ir)

    return requires


# local or publish
USE_CYTHON = True if glob.glob("pylibpcap/*.pyx") else False
ext = '.pyx' if USE_CYTHON else '.c'

ext_modules = [
    Extension(
        "{}/{}".format(directory, file.split(".")[0]).replace("/", "."),
        sources=["{}/{}".format(directory, file)],
        libraries=["m"],
        # include_dirs=["src"],
        extra_compile_args=["-lpcap"],
        extra_link_args=["-lpcap"],
    )
    for directory, dirs, files in os.walk("pylibpcap")
    for file in files if ext in file and ".pyc" not in file
]

ext_modules = cythonize(ext_modules) if USE_CYTHON else ext_modules


setup(
    name="python-libpcap",
    version="0.2.4",
    author="JanKinCai",
    author_email="jankincai12@gmail.com",
    maintainer="JanKinCai",
    maintainer_email="jankincai12@gmail.com",
    url="https://github.com/caizhengxin/python-libpcap",
    download_url="https://github.com/caizhengxin/python-libpcap.git",
    license="BSD",
    description="Cython libpcap",
    long_description=long_description,
    keywords=[
        "python-libpcap",
        "pylibpcap",
        "libpcap",
        "pcap",
        "python",
        "libpcap-python",
    ],
    zip_safe=False,
    packages=find_packages(),
    cmdclass={
        "build_ext": build_ext
    },
    ext_modules=ext_modules,
    install_requires=read_requirements("requirements/publish.txt"),
    entry_points={
        "console_scripts": [
            "mpcap = pylibpcap.command:main",
            "capture = pylibpcap.command:pylibpcap_sniff",
            "wpcap = pylibpcap.command:write_pcap_cli",
        ],
    },
    include_package_data=True,  # MANIFEST.in
    setup_requires=[
        "setuptools",
        "Cython",
    ],
    project_urls={
        "Documentation": "https://python-libpcap.readthedocs.io",
        "Source Code": "https://github.com/caizhengxin/python-libpcap",
    },
    platforms="Linux",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ],
)
